world
	hub = "Jp.GIAD2010"

	New()
		spawn() PlacePieces()

var/list/players = list()

var/list/waiting_list = list()

var/mob/black_player = null
var/mob/white_player = null

var/mob/player_turn = null

var/mob/piece/piece_selected = null

var/game_in_progress = FALSE

var/obj/turn_indicator = new /obj/turn_indicator()

proc/PieceChangeMenu(var/mob/piece/target)
	new /piece_menu(target, player_turn)

proc/CancelChange()
	for(var/turf/square/t) t.move_target = FALSE

	piece_selected = null

	if(player_turn.menu) del player_turn.menu
	player_turn.client.images.Cut()

proc/ChangePiece(mob/player, mob/target, typepath)
	for(var/turf/square/t) t.move_target = FALSE

	piece_selected = null

	var/obj/o = new /obj(target.loc)
	o.layer = FLY_LAYER
	o.text = "<font bgcolor=#FFFFFF> </font>"
	o.icon = 'board.dmi'
	flick("flash", o)
	sleep(9)

	new typepath(target.loc, target.side)
	del target

	sleep(9)

	del o

	EndTurn()

proc/EndTurn()
	for(var/turf/square/t)
		t.move_target = FALSE

	player_turn.client.images.Cut()
	piece_selected = null
	if(player_turn.menu) del player_turn.menu

	player_turn.client.screen -= turn_indicator

	if(player_turn == black_player)
		player_turn = white_player
	else
		player_turn = black_player

	player_turn.client.screen += turn_indicator

	var/result = CheckWin()
	if(result)
		Victory(result)

proc/CheckWin()
	var/blackcount = 0
	var/whitecount = 0

	var/area/a = locate(/area/blackside)
	for(var/mob/piece/p in a)
		if(p.side == "b") blackcount++
		if(p.side == "w") whitecount++

	if(whitecount > blackcount) return "w"

	blackcount = 0
	whitecount = 0

	a = locate(/area/whiteside)
	for(var/mob/piece/p in a)
		if(p.side == "b") blackcount++
		if(p.side == "w") whitecount++

	if(blackcount > whitecount) return "b"

	return ""

proc/CheckGameStart()
	if(!game_in_progress && waiting_list.len >= 2)
		StartGame()

proc/Victory(result)
	if(result == "w")
		world << "[white_player.name] (white) wins!"
	else
		world << "[black_player.name] (black) wins!"

	if(player_turn.menu) del player_turn.menu

	white_player.client.images.Cut()
	black_player.client.images.Cut()

	black_player.client.screen.Cut()
	white_player.client.screen.Cut()

	black_player.client.screen += new /obj/button/queue
	white_player.client.screen += new /obj/button/queue

	black_player.client.screen += new /obj/button/help
	white_player.client.screen += new /obj/button/help

	black_player.client.dir = NORTH

	black_player = null
	white_player = null
	player_turn = null
	piece_selected = null
	game_in_progress = FALSE

	CheckGameStart()

proc/StartGame()
	game_in_progress = TRUE

	var/list/players = list(waiting_list[1], waiting_list[2])

	waiting_list.Cut(1, 3)

	black_player = pick(players)

	players -= black_player
	white_player = players[1]

	world << "And the game begins!"
	world << "[white_player.name] (white) vs [black_player.name] (black)"

	for(var/mob/piece/p) del p
	PlacePieces()

	black_player.client.dir = SOUTH

	player_turn = white_player

	white_player.side = "w"
	black_player.side = "b"

	black_player.client.screen.Cut()
	white_player.client.screen.Cut()

	black_player.client.screen += new /obj/button/surrender
	white_player.client.screen += new /obj/button/surrender

	white_player.client.screen += new /obj/button/help
	black_player.client.screen += new /obj/button/help

	player_turn.client.screen += turn_indicator

proc/LineMove(xpos, ypos, xdir, ydir, side)
	var/list/ret = list()

	xpos+=xdir
	ypos+=ydir

	while(1)
		var/turf/t = BoardPos(xpos, ypos)
		if(!t) break

		var/mob/piece/p = locate(/mob/piece) in t
		if(p)
			if(p.side != side) ret += t
			break

		ret += t
		xpos+=xdir
		ypos+=ydir

	return ret

proc/LocToBoard(turf/t)
	if(t.x < 3 || t.y < 3 || t.x > 10 || t.y > 10) return list(-1, -1)

	return list(t.x - 2, t.y - 2)

proc/BoardPos(n, m)
	if(n < 1 || n > 8 || m < 1 || m > 8) return null

	return locate(n + 2, m + 2, 1)

proc/PlacePieces()
	for(var/i = 1; i < 9; i++)
		new /mob/piece/pawn(BoardPos(i, 2), "w")
		new /mob/piece/pawn(BoardPos(i, 7), "b")

	new /mob/piece/rook(BoardPos(1, 1), "w")
	new /mob/piece/rook(BoardPos(8, 1), "w")

	new /mob/piece/rook(BoardPos(1, 8), "b")
	new /mob/piece/rook(BoardPos(8, 8), "b")

	new /mob/piece/knight(BoardPos(2, 1), "w")
	new /mob/piece/knight(BoardPos(7, 1), "w")

	new /mob/piece/knight(BoardPos(2, 8), "b")
	new /mob/piece/knight(BoardPos(7, 8), "b")

	new /mob/piece/bishop(BoardPos(3, 1), "w")
	new /mob/piece/bishop(BoardPos(6, 1), "w")

	new /mob/piece/bishop(BoardPos(3, 8), "b")
	new /mob/piece/bishop(BoardPos(6, 8), "b")

	new /mob/piece/king(BoardPos(5, 1), "w")
	new /mob/piece/queen(BoardPos(4, 1), "w")

	new /mob/piece/king(BoardPos(5, 8), "b")
	new /mob/piece/queen(BoardPos(4, 8), "b")

turf
	text = ""
	name = ""

	square
		var/move_target = FALSE
		icon = 'board.dmi'

		Click()
			if(player_turn && player_turn.menu) return

			var/mob/piece/p = locate(/mob/piece) in src

			if(p)
				p.Click()
				return

			if(move_target)
				piece_selected.loc = src
				piece_selected.has_moved = TRUE
				EndTurn()

		black
			text = "<font bgcolor=#996600> </font>"
			icon_state = "black"

		white
			text = "<font bgcolor=#CCCCCC> </font>"
			icon_state = "white"

mob
	density = 0

	text = ""

	var
		side
		piece_menu/menu

	verb/say(t as text)
		set hidden = 1
		world << "[name]: [t]"

	verb/JoinGame()
		set hidden = 1
		if(!(src in waiting_list))
			waiting_list += src
			src << "You're in the queue!"
			src << "Position: [waiting_list.Find(src)]"
		else
			src << "You're already in the queue"

		CheckGameStart()

	verb/LeaveQueue()
		set hidden = 1
		if(src in waiting_list)
			waiting_list -= src
			src << "You've been removed from the queue"
		else
			src << "You're not in the queue"

	Login()
		. = ..()
		world << "[name] has joined the server"
		loc = null
		client.eye = locate(1, 1, 1)

		client.screen += new /obj/button/queue
		client.screen += new /obj/button/help

		players += src

	Logout()
		if(game_in_progress && (white_player == src || black_player == src))
			world << "[name] ([side=="w"?"white":"black"]) has abandoned the game"

			Victory(side=="w"?"b":"w")

		players -= src

		. = ..()

	Stat()
		if(game_in_progress)
			statpanel("Game")
			stat("[white_player.name] (white) vs [black_player.name] (black)")

		statpanel("Players")
		stat(players)

		if(waiting_list.len)
			statpanel("Waiting List")
			stat(waiting_list)

	Move()
		if(client) return 0
		return ..()

	piece
		icon = 'pieces.dmi'
		animate_movement = NO_STEPS

		var/has_moved = FALSE

		New(loc, s)
			. = ..()
			side = s
			icon_state = s + "_" + icon_state

			if(s == "w")
				text = "<font color=#ffffff>" + text + "</font>"
			else
				text = "<font color=#000000>" + text + "</font>"

		Click()
			if(player_turn && player_turn.menu) return

			if(usr == player_turn)
				player_turn.client.images.Cut()

				if(loc:move_target && side != piece_selected.side)
					piece_selected.loc = loc
					piece_selected.has_moved = TRUE
					loc = null

					EndTurn()
					return

				if(usr.side != side) return

				if(loc:move_target && side == piece_selected.side && istype(piece_selected, /mob/piece/king))
					PieceChangeMenu(src)
					return

				for(var/turf/square/t)
					t.move_target = FALSE

				if(piece_selected == src)
					piece_selected = null
					return

				var/image/i = image('board.dmi', src, "selbox", OBJ_LAYER)
				i.text = "<font bgcolor=#0000CC> </font>"
				player_turn << i

				piece_selected = src

				for(var/turf/square/t in FindLegalMoves())
					i = image('board.dmi', t, "highlight", OBJ_LAYER)
					i.text = "<font bgcolor=#0000ff> </font>"

					player_turn << i
					t.move_target = TRUE

		proc
			FindLegalMoves()
				return list()

		pawn
			icon_state = "pawn"

			text = "p"

			FindLegalMoves()
				var/list/ret = list()

				var/list/b_loc = LocToBoard(loc)
				if(b_loc[1] < 0) return

				var/turf/forward = BoardPos(b_loc[1], b_loc[2] + (side=="w"?1:-1))
				if(forward)
					if(!(locate(/mob/piece) in forward))
						ret += forward

						if(!has_moved)
							forward = BoardPos(b_loc[1], b_loc[2] + (side=="w"?2:-2))
							if(forward)
								if(!(locate(/mob/piece) in forward))
									ret += forward

				forward = BoardPos(b_loc[1] + 1, b_loc[2] + (side=="w"?1:-1))
				if(forward)
					var/mob/piece/p = locate(/mob/piece) in forward
					if(p && p.side != side) ret += forward

				forward = BoardPos(b_loc[1] - 1, b_loc[2] + (side=="w"?1:-1))
				if(forward)
					var/mob/piece/p = locate(/mob/piece) in forward
					if(p && p.side != side) ret += forward

				return ret

		rook
			icon_state = "rook"
			text = "R"

			FindLegalMoves()
				var/list/ret = list()

				var/list/b_loc = LocToBoard(loc)
				if(b_loc[1] < 0) return

				ret += LineMove(b_loc[1], b_loc[2], 1, 0, side)
				ret += LineMove(b_loc[1], b_loc[2], -1, 0, side)
				ret += LineMove(b_loc[1], b_loc[2], 0, 1, side)
				ret += LineMove(b_loc[1], b_loc[2], 0, -1, side)

				return ret

		bishop
			icon_state = "bishop"
			text = "B"

			FindLegalMoves()
				var/list/ret = list()

				var/list/b_loc = LocToBoard(loc)
				if(b_loc[1] < 0) return

				ret += LineMove(b_loc[1], b_loc[2], 1, 1, side)
				ret += LineMove(b_loc[1], b_loc[2], -1, 1, side)
				ret += LineMove(b_loc[1], b_loc[2], 1, -1, side)
				ret += LineMove(b_loc[1], b_loc[2], -1, -1, side)

				return ret

		knight
			icon_state = "knight"
			text = "N"

			FindLegalMoves()
				var/list/ret = list()
				var/list/b_loc = LocToBoard(loc)
				if(b_loc[1] < 0) return

				var/turf/t = BoardPos(b_loc[1] + 2, b_loc[2] + 1)
				var/mob/piece/p = locate(/mob/piece) in t
				if(!p || p.side != side) ret += t

				t = BoardPos(b_loc[1] + 1, b_loc[2] + 2)
				p = locate(/mob/piece) in t
				if(!p || p.side != side) ret += t

				t = BoardPos(b_loc[1] + 2, b_loc[2] - 1)
				p = locate(/mob/piece) in t
				if(!p || p.side != side) ret += t

				t = BoardPos(b_loc[1] - 1, b_loc[2] + 2)
				p = locate(/mob/piece) in t
				if(!p || p.side != side) ret += t

				t = BoardPos(b_loc[1] - 2, b_loc[2] + 1)
				p = locate(/mob/piece) in t
				if(!p || p.side != side) ret += t

				t = BoardPos(b_loc[1] + 1, b_loc[2] - 2)
				p = locate(/mob/piece) in t
				if(!p || p.side != side) ret += t

				t = BoardPos(b_loc[1] - 2, b_loc[2] - 1)
				p = locate(/mob/piece) in t
				if(!p || p.side != side) ret += t

				t = BoardPos(b_loc[1] - 1, b_loc[2] - 2)
				p = locate(/mob/piece) in t
				if(!p || p.side != side) ret += t

				return ret

		queen
			icon_state = "queen"
			text = "Q"

			FindLegalMoves()
				var/list/ret = list()

				var/list/b_loc = LocToBoard(loc)
				if(b_loc[1] < 0) return

				ret += LineMove(b_loc[1], b_loc[2], 1, 0, side)
				ret += LineMove(b_loc[1], b_loc[2], -1, 0, side)
				ret += LineMove(b_loc[1], b_loc[2], 0, 1, side)
				ret += LineMove(b_loc[1], b_loc[2], 0, -1, side)

				ret += LineMove(b_loc[1], b_loc[2], 1, 1, side)
				ret += LineMove(b_loc[1], b_loc[2], -1, 1, side)
				ret += LineMove(b_loc[1], b_loc[2], 1, -1, side)
				ret += LineMove(b_loc[1], b_loc[2], -1, -1, side)

				return ret

		king
			icon_state = "king"
			text = "K"

			FindLegalMoves()
				var/list/ret = list()

				for(var/turf/t in orange(1, loc))
					var/list/b_loc = LocToBoard(t)
					if(b_loc[1] > 0) ret += t

				return ret

area
	text = ""

	icon = 'board.dmi'
	whiteside
		icon_state = "whiteside"

	blackside
		icon_state = "blackside"

piece_menu
	var/mob/piece/target
	var/mob/player

	var/obj/button
		pawn_button
		bishop_button
		knight_button
		rook_button
		no/cancel_button

	proc
		Cancel()
			CancelChange()

		Selected(obj/button/b)
			if(b == pawn_button)
				ChangePiece(player, target, /mob/piece/pawn)

			if(b == knight_button)
				ChangePiece(player, target, /mob/piece/knight)

			if(b == bishop_button)
				ChangePiece(player, target, /mob/piece/bishop)

			if(b == rook_button)
				ChangePiece(player, target, /mob/piece/rook)

	Del()
		del pawn_button
		del bishop_button
		del knight_button
		del rook_button
		del cancel_button
		. = ..()

	New(mob/piece/target, mob/player)
		src.target = target
		src.player = player

		var/image/i = image('board.dmi', target, "selbox", OBJ_LAYER)
		i.text = "<font bgcolor=#0000CC> </font>"
		player_turn << i

		player.menu = src

		var/mob/piece/p

		cancel_button = new /obj/button/no
		cancel_button.screen_loc = "NORTH, EAST"
		cancel_button.menu_ref = src

		p = new /mob/piece/pawn(pawn_button, player.side)
		pawn_button = new /obj/button
		pawn_button.screen_loc = "NORTH-1, EAST"
		pawn_button.text = "<font bgcolor=#cccccc>[p.text]</font>"
		pawn_button.overlays += p
		pawn_button.menu_ref = src

		p = new /mob/piece/knight(null, player.side)
		knight_button = new /obj/button
		knight_button.screen_loc = "NORTH-2, EAST"
		knight_button.overlays += p
		knight_button.text = "<font bgcolor=#cccccc>[p.text]</font>"
		knight_button.menu_ref = src

		p = new /mob/piece/bishop(null, player.side)
		bishop_button = new /obj/button
		bishop_button.screen_loc = "NORTH-3, EAST"
		bishop_button.overlays += p
		bishop_button.text = "<font bgcolor=#cccccc>[p.text]</font>"
		bishop_button.menu_ref = src

		p = new /mob/piece/rook(null, player.side)
		rook_button = new /obj/button
		rook_button.screen_loc = "NORTH-4, EAST"
		rook_button.overlays += p
		rook_button.text = "<font bgcolor=#cccccc>[p.text]</font>"
		rook_button.menu_ref = src

		player.client.screen += cancel_button
		player.client.screen += pawn_button
		player.client.screen += knight_button
		player.client.screen += bishop_button
		player.client.screen += rook_button

obj
	turn_indicator
		icon = 'turn_indicator.png'
		text = "|/-\\"
		screen_loc = "NORTH-1, WEST+4"

	overlay
		icon = 'board.dmi'

		whiteside_edge
			icon_state = "whiteside"

		blackside_edge
			icon_state = "blackside"

		selbox
			icon_state = "selbox"

		highlight
			icon_state = "highlight"

		flash
			icon_state = "flash"

	button
		icon = 'buttons.dmi'
		icon_state = "button"

		var/piece_menu/menu_ref

		Click()
			if(menu_ref)
				if(usr == menu_ref.player)
					menu_ref.Selected(src)

		surrender
			icon_state = "surrender"
			screen_loc = "NORTH,WEST"
			text = "S"

			Click()
				if(game_in_progress && (usr == white_player || usr == black_player))
					var/result = alert(usr, "Do you want to surrender?", "Surrender!", "Yes", "No")
					if(result == "Yes")
						world << "[usr.name] surrenders!"

						Victory(usr == white_player?"b":"w")

		queue
			icon_state = "queue"
			screen_loc = "NORTH,WEST"
			text = "+"

			Click()
				if(icon_state == "queue")
					icon_state = "dequeue"
					text = "-"
					usr.JoinGame()
				else
					icon_state = "queue"
					text = "+"
					usr.LeaveQueue()

		yes
			icon_state = "yes"
			text = "Y"

		no
			icon_state = "no"
			text = "<font bgcolor=#cccccc color=#ffffff>X</font>"

			Click()
				if(menu_ref)
					if(usr == menu_ref.player)
						menu_ref.Cancel()

		help
			icon_state = "help"
			text = "?"

			screen_loc = "SOUTH, WEST"

			Click()
				usr << browse(file2text('help.txt'), "window=help")